@@ -12,7 +12,9 @@ use Raisin::Routes;
 
 use Raisin::Util;
 
-our $VERSION = '0.27';
+use constant DEFAULT_SERIALIZER => 'Raisin::Plugin::Format::TEXT';
+
+our $VERSION = '0.28';
 
 sub new {
     my ($class, %args) = @_;
@@ -84,7 +86,7 @@ sub psgi {
 
     # Diffrent for each response
     my $req = $self->req(Raisin::Request->new($self, $env));
-    my $res = $self->res(Raisin::Response->new);
+    my $res = $self->res(Raisin::Response->new($self));
 
     # Build API docs if needed
     if ($self->can('build_api_docs')) {
@@ -104,8 +106,8 @@ sub psgi {
 
     eval {
         foreach my $route (@$routes) {
+            # Validate code variable
             my $code = $route->code;
-
             if (!$code || (ref($code) && ref($code) ne 'CODE')) {
                 croak 'Invalid endpoint for ' . $req->path;
             }
@@ -126,7 +128,7 @@ sub psgi {
             $req->set_named_params($route->named);
 
             # Populate and validate declared params
-            if (not $req->populate_params) {
+            if (not $req->prepare_params) {
                 carp '* ' . 'INVALID PARAMS! ' x 5;
                 $res->render_error(400, 'Invalid params!');
                 last;
@@ -135,36 +137,16 @@ sub psgi {
             # HOOK After validation
             $self->hook('after_validation')->($self);
 
-            # Set default content type
-            $res->content_type($self->default_content_type);
-
             # Eval code
             my $data = $code->($req->declared_params);
 
-            # Format plugins
-            if (ref $data && $self->can('serialize')) {
-                my $format = $route->format || $req->header('Accept');
-
-                if (my $serializer = Raisin::Util::detect_serializer($format)) {
-                    my $class = 'Raisin::Plugin::Format::' . uc($serializer);
-                    Plack::Util::load_class($class);
-
-                    {
-                        no strict 'refs';
-                        $data = "${class}::serialize"->($data);
-                    }
-
-                    $res->$serializer;
-                }
-                else {
-                    $data = $self->serialize($data);
-                }
-            }
-
             if (defined $data) {
                 # Handle delayed response
                 return $data if ref($data) eq 'CODE'; # TODO check delayed responses
-                $res->render($data) if not $res->rendered;
+
+                # Detect output format
+                my $format = $route->format || $req->header('Accept');
+                $res->render($format, $data) if not $res->rendered;
             }
 
             # HOOK After
@@ -209,12 +191,10 @@ sub api_version {
 
 sub api_format {
     my ($self, $name) = @_;
-    $name = $name =~ /\+/x ? $name : "Format::$name";
+    $name = $name =~ /\+/x ? $name : "Format::${\uc($name)}";
     $self->load_plugin($name);
 }
 
-sub default_content_type { shift->{default_content_type} || 'text/plain' }
-
 # Request and Response and shortcuts
 sub req {
     my ($self, $req) = @_;
@@ -246,7 +226,7 @@ __END__
 
 =head1 NAME
 
-Raisin - REST-like API micro-framework for Perl.
+Raisin - REST-like API web micro-framework for Perl.
 
 =head1 SYNOPSIS
 
@@ -320,7 +300,7 @@ Raisin - REST-like API micro-framework for Perl.
 
 =head1 DESCRIPTION
 
-Raisin is a REST-like API micro-framework for Perl.
+Raisin is a REST-like API web micro-framework for Perl.
 It's designed to run on Plack, providing a simple DSL to easily develop RESTful APIs.
 It was inspired by L<Grape|https://github.com/intridea/grape>.
 
@@ -435,7 +415,7 @@ Loads a plugin from C<Raisin::Plugin::Format> namespace.
 
 Already exists L<Raisin::Plugin::Format::JSON> and L<Raisin::Plugin::Format::YAML>.
 
-    api_format 'JSON';
+    api_format 'json';
 
 =head2 plugin
 
@@ -463,7 +443,7 @@ In C<RaisinApp.pm>:
 
     use Raisin::API;
 
-    api_format 'JSON';
+    api_format 'json';
 
     mount 'RaisinApp::User';
     mount 'RaisinApp::Host';
@@ -481,7 +461,7 @@ GET, POST and PUT parameters, along with any named parameters you specify in
 your route strings.
 
 Parameters are automatically populated from the request body on POST and PUT
-for form input, JSON and YAML content-types.
+for form input, C<JSON> and C<YAML> content-types.
 
 In the case of conflict between either of:
 
@@ -619,31 +599,33 @@ Steps 3 and 4 only happen if validation succeeds.
 By default, Raisin supports C<YAML>, C<JSON>, and C<TEXT> content-types.
 The default format is C<TEXT>.
 
-Response format can be determined by Accept header.
+Response format can be determined by Accept header or route extension.
 
 Serialization takes place automatically. For example, you do not have to call
 C<encode_json> in each C<JSON> API implementation.
 
 Your API can declare which types to support by using C<api_format>.
 
-    api_format 'JSON';
+    api_format 'json';
 
 Custom formatters for existing and additional types can be defined with a
 L<Raisin::Plugin::Format>.
 
-=head2 JSON
+=over
+
+=item JSON
 
 Call C<JSON::encode_json> and C<JSON::decode_json>.
 
-=head2 YAML
+=item YAML
 
 Call C<YAML::Dump> and C<JSON::Load>.
 
-=head2 TEXT
+=item TEXT
 
 Call C<Data::Dumper-E<gt>Dump> if output data is not a string.
 
-The order for choosing the format is the following.
+=back
 
 The order for choosing the format is the following.
 
@@ -651,7 +633,11 @@ The order for choosing the format is the following.
 
 =item *
 
-Use the value of ther C<Accept> header.
+Use the route extension.
+
+=item *
+
+Use the value of the C<Accept> header.
 
 =item *
 
